#include "parameterdump.h"
#include <QTime>
#include <QDate>
#include <QFileDialog>
#include <QSettings>

ParameterDump::ParameterDump(QWidget *parent) :
    QWidget(parent)
{
    readSettings();

    this->buildTabParameterDump();

    paulCommonCRC = new PaulCommonCRC();

    kCoefficientsRecordFileEn = false;
    fBinsMasksRecordFileEn = false;
    kCoefficientsRecordFile = new QFile();
    fBinsMasksRecordFile = new QFile();

    this->setLayout(layout_parameters);
}

ParameterDump::~ParameterDump()
{
    writeSettings();
}

void ParameterDump::buildTabParameterDump()
{
    buildCOMM();
    buildNORM();
    buildBURST();
    buildSBM1();
    buildSBM2();
    buildDUMP();

    layout_parameters = new QGridLayout();

    layout_parameters->addWidget(groupbox_COMM, 0, 0, 1, 1);
    layout_parameters->addWidget(groupbox_NORM, 0, 1, 1, 1);
    layout_parameters->addWidget(groupbox_BURST, 1, 0, 1, 1);
    layout_parameters->addWidget(groupbox_SBM1, 1 , 1, 1, 1);
    layout_parameters->addWidget(groupbox_SBM2, 2, 0, 1, 1);
    layout_parameters->addWidget(groupbox_dump, 2, 1, 1, 1);

    layout_parameters->setRowStretch(2, 1);
    layout_parameters->setColumnStretch(2, 1);
}

void ParameterDump::buildCOMM()
{
    groupbox_COMM = new QGroupBox(tr("COMMON_PAR"));
    layout_COMM = new QGridLayout();

    sy_lfr_bw = new QLabel("sy_lfr_bw ");
    sy_lfr_sp0 = new QLabel("sy_lfr_sp0 ");
    sy_lfr_sp1 = new QLabel("sy_lfr_sp1 ");
    sy_lfr_r0 = new QLabel("sy_lfr_r0 ");
    sy_lfr_r1 = new QLabel("sy_lfr_r1 ");
    sy_lfr_r2 = new QLabel("sy_lfr_r2 ");

    spinbox_sy_lfr_bw = new QSpinBox();
    spinbox_sy_lfr_sp0 = new QSpinBox();
    spinbox_sy_lfr_sp1 = new QSpinBox();
    spinbox_sy_lfr_r0 = new QSpinBox();
    spinbox_sy_lfr_r1 = new QSpinBox();
    spinbox_sy_lfr_r2 = new QSpinBox();

    button_loadCommon = new QPushButton(tr("LOAD_COMM"));

    button_loadCommon->setEnabled(false);

    spinbox_sy_lfr_bw->setRange(0, 1);
    spinbox_sy_lfr_sp0->setRange(0, 1);
    spinbox_sy_lfr_sp1->setRange(0, 1);
    spinbox_sy_lfr_r0->setRange(0, 1);
    spinbox_sy_lfr_r1->setRange(0, 1);
    spinbox_sy_lfr_r2->setRange(0, 1);

    spinbox_sy_lfr_bw->setValue(1);
    spinbox_sy_lfr_sp0->setValue(0);
    spinbox_sy_lfr_sp1->setValue(0);
    spinbox_sy_lfr_r0->setValue(0);
    spinbox_sy_lfr_r1->setValue(0);
    spinbox_sy_lfr_r2->setValue(0);

    layout_COMM->addWidget(button_loadCommon, 0, 0, 1, 2);
    layout_COMM->addWidget(sy_lfr_bw, 1, 0, 1, 1);
    layout_COMM->addWidget(sy_lfr_sp0, 2, 0, 1, 1);
    layout_COMM->addWidget(sy_lfr_sp1, 3, 0, 1, 1);
    layout_COMM->addWidget(sy_lfr_r0, 4, 0, 1, 1);
    layout_COMM->addWidget(sy_lfr_r1, 5, 0, 1, 1);
    layout_COMM->addWidget(sy_lfr_r2, 6, 0, 1, 1);

    layout_COMM->addWidget(spinbox_sy_lfr_bw, 1, 1, 1, 1);
    layout_COMM->addWidget(spinbox_sy_lfr_sp0, 2, 1, 1, 1);
    layout_COMM->addWidget(spinbox_sy_lfr_sp1, 3, 1, 1, 1);
    layout_COMM->addWidget(spinbox_sy_lfr_r0, 4, 1, 1, 1);
    layout_COMM->addWidget(spinbox_sy_lfr_r1, 5, 1, 1, 1);
    layout_COMM->addWidget(spinbox_sy_lfr_r2, 6, 1, 1, 1);

    layout_COMM->setColumnStretch(2, 1);
    layout_COMM->setRowStretch(7, 1);

    groupbox_COMM->setLayout(layout_COMM);

    connect(this->button_loadCommon, SIGNAL(clicked()), this, SLOT(sendLoadCommon()));
}

void ParameterDump::buildNORM()
{
    groupbox_NORM = new QGroupBox(tr("NORMAL_PAR"));
    layout_NORM = new QGridLayout();

    sy_lfr_n_swf_l = new QLabel("sy_lfr_n_swf_l ");
    sy_lfr_n_swf_p = new QLabel("sy_lfr_n_swf_p ");
    sy_lfr_n_asm_p = new QLabel("sy_lfr_n_asm_p ");
    sy_lfr_n_bp_p0 = new QLabel("sy_lfr_n_bp_p0 ");
    sy_lfr_n_bp_p1 = new QLabel("sy_lfr_n_bp_p1 ");
    sy_lfr_n_cwf_long_f3 = new QLabel("sy_lfr_n_cwf_long_f3 ");

    spinbox_sy_lfr_n_swf_l = new QSpinBox();
    spinbox_sy_lfr_n_swf_p = new QSpinBox();
    spinbox_sy_lfr_n_asm_p = new QSpinBox();
    spinbox_sy_lfr_n_bp_p0 = new QSpinBox();
    spinbox_sy_lfr_n_bp_p1 = new QSpinBox();
    spinbox_sy_lfr_n_cwf_long_f3 = new QSpinBox();

    button_loadNormal = new QPushButton(tr("LOAD_NORM"));

    button_loadNormal->setEnabled(false);

    spinbox_sy_lfr_n_swf_l->setRange(0, 65535);
    spinbox_sy_lfr_n_swf_p->setRange(0, 65535);
    spinbox_sy_lfr_n_asm_p->setRange(0, 65535);
    spinbox_sy_lfr_n_bp_p0->setRange(0, 255);
    spinbox_sy_lfr_n_bp_p1->setRange(0, 255);
    spinbox_sy_lfr_n_cwf_long_f3->setRange(0, 1);

    spinbox_sy_lfr_n_swf_l->setValue(2048);
    spinbox_sy_lfr_n_swf_p->setValue(300);
    spinbox_sy_lfr_n_asm_p->setValue(3600);
    spinbox_sy_lfr_n_bp_p0->setValue(4);
    spinbox_sy_lfr_n_bp_p1->setValue(20);
    spinbox_sy_lfr_n_cwf_long_f3->setValue(0);

    layout_NORM->addWidget(button_loadNormal, 0, 0, 1, 2);
    layout_NORM->addWidget(sy_lfr_n_swf_l, 1, 0, 1, 1);
    layout_NORM->addWidget(sy_lfr_n_swf_p, 2, 0, 1, 1);
    layout_NORM->addWidget(sy_lfr_n_asm_p, 3, 0, 1, 1);
    layout_NORM->addWidget(sy_lfr_n_bp_p0, 4, 0, 1, 1);
    layout_NORM->addWidget(sy_lfr_n_bp_p1, 5, 0, 1, 1);
    layout_NORM->addWidget(sy_lfr_n_cwf_long_f3, 6, 0, 1, 1);

    layout_NORM->addWidget(spinbox_sy_lfr_n_swf_l, 1, 1, 1, 1);
    layout_NORM->addWidget(spinbox_sy_lfr_n_swf_p, 2, 1, 1, 1);
    layout_NORM->addWidget(spinbox_sy_lfr_n_asm_p, 3, 1, 1, 1);
    layout_NORM->addWidget(spinbox_sy_lfr_n_bp_p0, 4, 1, 1, 1);
    layout_NORM->addWidget(spinbox_sy_lfr_n_bp_p1, 5, 1, 1, 1);
    layout_NORM->addWidget(spinbox_sy_lfr_n_cwf_long_f3, 6, 1, 1, 1);

    layout_NORM->setColumnStretch(2, 1);
    layout_NORM->setRowStretch(7, 1);

    groupbox_NORM->setLayout(layout_NORM);

    connect(this->button_loadNormal, SIGNAL(clicked()), this, SLOT(sendLoadNormal()));
}

void ParameterDump::buildBURST()
{
    groupbox_BURST = new QGroupBox(tr("BURST_PAR"));
    layout_BURST = new QGridLayout();

    sy_lfr_b_bp_p0 = new QLabel("sy_lfr_b_bp_p0 ");
    sy_lfr_b_bp_p1 = new QLabel("sy_lfr_b_bp_p1 ");

    spinbox_sy_lfr_b_bp_p0 = new QSpinBox();
    spinbox_sy_lfr_b_bp_p1 = new QSpinBox();

    button_loadBurst = new QPushButton(tr("LOAD_BURST"));

    button_loadBurst->setEnabled(false);

    spinbox_sy_lfr_b_bp_p0->setRange(0, 255);
    spinbox_sy_lfr_b_bp_p1->setRange(0, 255);
    spinbox_sy_lfr_b_bp_p0->setValue(1);
    spinbox_sy_lfr_b_bp_p1->setValue(5);

    layout_BURST->addWidget(button_loadBurst, 0, 0, 1, 2);
    layout_BURST->addWidget(sy_lfr_b_bp_p0, 1, 0, 1, 1);
    layout_BURST->addWidget(sy_lfr_b_bp_p1, 2, 0, 1, 1);

    layout_BURST->addWidget(spinbox_sy_lfr_b_bp_p0, 1, 1, 1, 1);
    layout_BURST->addWidget(spinbox_sy_lfr_b_bp_p1, 2, 1, 1, 1);

    layout_BURST->setColumnStretch(2, 1);
    layout_BURST->setRowStretch(3, 1);

    groupbox_BURST->setLayout(layout_BURST);

    connect(this->button_loadBurst, SIGNAL(clicked()), this, SLOT(sendLoadBurst()));
}

void ParameterDump::buildSBM1()
{
    groupbox_SBM1 = new QGroupBox(tr("SBM1_PAR"));
    layout_SBM1 = new QGridLayout();

    sy_lfr_s1_bp_p0 = new QLabel("sy_lfr_s1_bp_p0 ");
    sy_lfr_s1_bp_p1 = new QLabel("sy_lfr_s1_bp_p1 ");

    spinbox_sy_lfr_s1_bp_p0 = new QSpinBox();
    spinbox_sy_lfr_s1_bp_p1 = new QSpinBox();

    button_loadSBM1 = new QPushButton(tr("LOAD_SBM1"));

    button_loadSBM1->setEnabled(false);

    spinbox_sy_lfr_s1_bp_p0->setRange(0, 255);
    spinbox_sy_lfr_s1_bp_p0->setValue(1);
    spinbox_sy_lfr_s1_bp_p1->setRange(0, 255);
    spinbox_sy_lfr_s1_bp_p1->setValue(1);

    layout_SBM1->addWidget(button_loadSBM1, 0, 0, 1, 2);
    layout_SBM1->addWidget(sy_lfr_s1_bp_p0, 1, 0, 1, 1);
    layout_SBM1->addWidget(sy_lfr_s1_bp_p1, 2, 0, 1, 1);

    layout_SBM1->addWidget(spinbox_sy_lfr_s1_bp_p0, 1, 1, 1, 1);
    layout_SBM1->addWidget(spinbox_sy_lfr_s1_bp_p1, 2, 1, 1, 1);

    layout_SBM1->setColumnStretch(2, 1);
    layout_SBM1->setRowStretch(3, 1);

    groupbox_SBM1->setLayout(layout_SBM1);

    connect(this->button_loadSBM1, SIGNAL(clicked()), this, SLOT(sendLoadSBM1()));
}

void ParameterDump::buildSBM2()
{
    groupbox_SBM2 = new QGroupBox(tr("SBM2_PAR"));
    layout_SBM2 = new QGridLayout();

    sy_lfr_s2_bp_p0 = new QLabel("sy_lfr_s2_bp_p0 ");
    sy_lfr_s2_bp_p1 = new QLabel("sy_lfr_s2_bp_p1 ");

    spinbox_sy_lfr_s2_bp_p0 = new QSpinBox();
    spinbox_sy_lfr_s2_bp_p1 = new QSpinBox();

    button_loadSBM2 = new QPushButton(tr("LOAD_SBM2"));

    button_loadSBM2->setEnabled(false);

    spinbox_sy_lfr_s2_bp_p0->setRange(0, 255);
    spinbox_sy_lfr_s2_bp_p1->setRange(0, 255);
    spinbox_sy_lfr_s2_bp_p0->setValue(1);
    spinbox_sy_lfr_s2_bp_p1->setValue(5);

    layout_SBM2->addWidget(button_loadSBM2, 0, 0, 1, 2);

    layout_SBM2->addWidget(sy_lfr_s2_bp_p0, 1, 0, 1, 1);
    layout_SBM2->addWidget(sy_lfr_s2_bp_p1, 2, 0, 1, 1);

    layout_SBM2->addWidget(spinbox_sy_lfr_s2_bp_p0, 1, 1, 1, 1);
    layout_SBM2->addWidget(spinbox_sy_lfr_s2_bp_p1, 2, 1, 1, 1);

    layout_SBM2->setColumnStretch(2, 1);
    layout_SBM2->setRowStretch(3, 1);

    groupbox_SBM2->setLayout(layout_SBM2);

    connect(this->button_loadSBM2, SIGNAL(clicked()), this, SLOT(sendLoadSBM2()));
}

void ParameterDump::buildDUMP()
{
    groupbox_dump = new QGroupBox(tr("DUMP"));
    layout_dump = new QGridLayout();

    button_parameterDump = new QPushButton(tr("DUMP_PAR"));
    button_kDump         = new QPushButton(tr("DUMP_KCOEFFICIENTS"));
    checkbox_kCoefficientsRecording = new QCheckBox("Store K coefficients");
    checkbox_fBinsMasksRecording = new QCheckBox("Store fbins masks");

    layout_dump->addWidget( button_parameterDump, 0, 0, 1, 1 );
    layout_dump->addWidget( button_kDump,         1, 0, 1, 1 );
    layout_dump->addWidget( checkbox_kCoefficientsRecording, 2, 0, 1, 1 );
    layout_dump->addWidget( checkbox_fBinsMasksRecording, 3, 0, 1, 1 );

    layout_dump->setColumnStretch(1, 1);
    layout_dump->setRowStretch(4, 1);

    groupbox_dump->setLayout(layout_dump);

    button_parameterDump->setEnabled(false);

    connect(this->button_parameterDump, SIGNAL(clicked()), this, SLOT(sendParameterDump()));
    connect(this->button_kDump,         SIGNAL(clicked()), this, SLOT(sendKDump()));
    connect(this->checkbox_kCoefficientsRecording, SIGNAL(stateChanged(int)),
            this, SLOT(storeKCoefficients(int)));
    connect(this->checkbox_fBinsMasksRecording, SIGNAL(stateChanged(int)),
            this, SLOT(storeFBinsMasks(int)));
}

void ParameterDump::updateParameterDump(TMPacketToRead *tmPacketToRead)
{

    Packet_TM_LFR_PARAMETER_DUMP_t *parameterPacket;

    parameterPacket = (Packet_TM_LFR_PARAMETER_DUMP_t *) tmPacketToRead->Value;

    // COMMON
    spinbox_sy_lfr_bw->setValue(  (parameterPacket->sy_lfr_common_parameters & 0x20 ) >> 5 );
    spinbox_sy_lfr_sp0->setValue( (parameterPacket->sy_lfr_common_parameters & 0x10 ) >> 4 );
    spinbox_sy_lfr_sp1->setValue( (parameterPacket->sy_lfr_common_parameters & 0x08 ) >> 3 );
    spinbox_sy_lfr_r0->setValue(  (parameterPacket->sy_lfr_common_parameters & 0x04 ) >> 2 );
    spinbox_sy_lfr_r1->setValue(  (parameterPacket->sy_lfr_common_parameters & 0x02 ) >> 1 );
    spinbox_sy_lfr_r2->setValue(  (parameterPacket->sy_lfr_common_parameters & 0x01 )      );

    // NORMAL
    spinbox_sy_lfr_n_swf_l->setValue(parameterPacket->sy_lfr_n_swf_l[0] * 256
            + parameterPacket->sy_lfr_n_swf_l[1]);
    spinbox_sy_lfr_n_swf_p->setValue(parameterPacket->sy_lfr_n_swf_p[0] * 256
            + parameterPacket->sy_lfr_n_swf_p[1]);
    spinbox_sy_lfr_n_asm_p->setValue(parameterPacket->sy_lfr_n_asm_p[0] * 256
            + parameterPacket->sy_lfr_n_asm_p[1]);
    spinbox_sy_lfr_n_bp_p0->setValue(parameterPacket->sy_lfr_n_bp_p0);
    spinbox_sy_lfr_n_bp_p1->setValue(parameterPacket->sy_lfr_n_bp_p1);
    spinbox_sy_lfr_n_cwf_long_f3->setValue(parameterPacket->sy_lfr_n_cwf_long_f3);

    // BURST
    spinbox_sy_lfr_b_bp_p0->setValue(parameterPacket->sy_lfr_b_bp_p0);
    spinbox_sy_lfr_b_bp_p1->setValue(parameterPacket->sy_lfr_b_bp_p1);

    // SBM1
    spinbox_sy_lfr_s1_bp_p0->setValue(parameterPacket->sy_lfr_s1_bp_p0);
    spinbox_sy_lfr_s1_bp_p1->setValue(parameterPacket->sy_lfr_s1_bp_p1);

    // SBM2
    spinbox_sy_lfr_s2_bp_p0->setValue(parameterPacket->sy_lfr_s2_bp_p0);
    spinbox_sy_lfr_s2_bp_p1->setValue(parameterPacket->sy_lfr_s2_bp_p1);

    if (fBinsMasksRecordFileEn == true)
    {
        processFBinsMasksDump( tmPacketToRead );
    }
}

void ParameterDump::sendParameterDump()
{
    Packet_TC_LFR_DUMP_PAR_t packet;
    unsigned char crcAsTwoBytes[2];

    packet.packetID[0] = (unsigned char) (TC_LFR_PACKET_ID >> 8);
    packet.packetID[1] = (unsigned char) (TC_LFR_PACKET_ID     );
    packet.packetSequenceControl[0] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL >> 8);
    packet.packetSequenceControl[1] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL     );
    packet.packetLength[0] = (unsigned char) (PACKET_LENGTH_TC_LFR_DUMP_PAR >> 8);
    packet.packetLength[1] = (unsigned char) (PACKET_LENGTH_TC_LFR_DUMP_PAR     );

    packet.ccsdsSecHeaderFlag_pusVersion_ack = 0x19;
    packet.serviceType = TC_TYPE_DEFAULT;
    packet.serviceSubType = TC_SUBTYPE_DUMP;
    packet.sourceID = SID_TC_RPW_INTERNAL;

    paulCommonCRC->GetCRCAsTwoBytes((unsigned char*) &packet, crcAsTwoBytes,
                                   PACKET_LENGTH_TC_LFR_DUMP_PAR + CCSDS_TC_TM_PACKET_OFFSET - 2);
    packet.crc[0] = crcAsTwoBytes[0];
    packet.crc[1] = crcAsTwoBytes[1];

    emit WriteSPWSig((char*) &packet, PACKET_LENGTH_TC_LFR_DUMP_PAR + CCSDS_TC_TM_PACKET_OFFSET,
                     CCSDS_NODE_ADDRESS, CCSDS_USER_APP);
}

void ParameterDump::sendKDump()
{
    Packet_TC_LFR_DUMP_KCOEFFICIENTS_t packet;
    unsigned char crcAsTwoBytes[2];

    if (kCoefficientsRecordFileEn == true)
    {
        buildKCoefficientsFileName();
    }

    packet.packetID[0] = (unsigned char) (TC_LFR_PACKET_ID >> 8);
    packet.packetID[1] = (unsigned char) (TC_LFR_PACKET_ID     );
    packet.packetSequenceControl[0] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL >> 8);
    packet.packetSequenceControl[1] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL     );
    packet.packetLength[0] = (unsigned char) (PACKET_LENGTH_TC_LFR_DUMP_KCOEFFICIENTS >> 8);
    packet.packetLength[1] = (unsigned char) (PACKET_LENGTH_TC_LFR_DUMP_KCOEFFICIENTS     );

    packet.ccsdsSecHeaderFlag_pusVersion_ack = 0x19;
    packet.serviceType = TC_TYPE_DEFAULT;
    packet.serviceSubType = TC_SUBTYPE_DUMP_K;
    packet.sourceID = SID_TC_RPW_INTERNAL;

    paulCommonCRC->GetCRCAsTwoBytes((unsigned char*) &packet, crcAsTwoBytes,
                                   PACKET_LENGTH_TC_LFR_DUMP_KCOEFFICIENTS + CCSDS_TC_TM_PACKET_OFFSET - 2);
    packet.crc[0] = crcAsTwoBytes[0];
    packet.crc[1] = crcAsTwoBytes[1];

    emit WriteSPWSig((char*) &packet, PACKET_LENGTH_TC_LFR_DUMP_KCOEFFICIENTS + CCSDS_TC_TM_PACKET_OFFSET,
                     CCSDS_NODE_ADDRESS, CCSDS_USER_APP);
}

void ParameterDump::sendLoadCommon()
{
    Packet_TC_LFR_LOAD_COMMON_PAR_t packet;
    unsigned char crcAsTwoBytes[2];

    packet.packetID[0] = (unsigned char) (TC_LFR_PACKET_ID >> 8);
    packet.packetID[1] = (unsigned char) (TC_LFR_PACKET_ID     );
    packet.packetSequenceControl[0] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL >> 8);
    packet.packetSequenceControl[1] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL     );
    packet.packetLength[0] = (unsigned char) (PACKET_LENGTH_TC_LFR_LOAD_COMMON_PAR >> 8);
    packet.packetLength[1] = (unsigned char) (PACKET_LENGTH_TC_LFR_LOAD_COMMON_PAR     );

    packet.ccsdsSecHeaderFlag_pusVersion_ack = TC_LFR_DATA_FIELD_HEADER0;
    packet.serviceType = TC_TYPE_DEFAULT;
    packet.serviceSubType = TC_SUBTYPE_LOAD_COMMON_PAR;
    packet.sourceID = SID_TC_RPW_INTERNAL;
    packet.spare = TC_LFR_LOAD_COMMON_PAR_SPARE;
    packet.bw_sp0_sp1_r0_r1_r2 = (spinbox_sy_lfr_bw->value() << 5 )
            + (spinbox_sy_lfr_sp0->value() << 4 )
            + (spinbox_sy_lfr_sp1->value() << 3 )
            + (spinbox_sy_lfr_r0->value()  << 2 )
            + (spinbox_sy_lfr_r1->value()  << 1 )
            + (spinbox_sy_lfr_r2->value()       );

    paulCommonCRC->GetCRCAsTwoBytes((unsigned char*) &packet, crcAsTwoBytes,
                                   PACKET_LENGTH_TC_LFR_LOAD_COMMON_PAR + CCSDS_TC_TM_PACKET_OFFSET - 2);
    packet.crc[0] = crcAsTwoBytes[0];
    packet.crc[1] = crcAsTwoBytes[1];

    emit WriteSPWSig((char*) &packet, PACKET_LENGTH_TC_LFR_LOAD_COMMON_PAR + CCSDS_TC_TM_PACKET_OFFSET,
                     CCSDS_NODE_ADDRESS, CCSDS_USER_APP);
}

void ParameterDump::sendLoadNormal()
{
    Packet_TC_LFR_LOAD_NORMAL_PAR_t packet;
    unsigned char crcAsTwoBytes[2];

    packet.packetID[0] = (unsigned char) (TC_LFR_PACKET_ID >> 8);
    packet.packetID[1] = (unsigned char) (TC_LFR_PACKET_ID     );
    packet.packetSequenceControl[0] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL >> 8);
    packet.packetSequenceControl[1] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL     );
    packet.packetLength[0] = (unsigned char) (PACKET_LENGTH_TC_LFR_LOAD_NORMAL_PAR >> 8);
    packet.packetLength[1] = (unsigned char) (PACKET_LENGTH_TC_LFR_LOAD_NORMAL_PAR     );

    packet.ccsdsSecHeaderFlag_pusVersion_ack = TC_LFR_DATA_FIELD_HEADER0;
    packet.serviceType = TC_TYPE_DEFAULT;
    packet.serviceSubType = TC_SUBTYPE_LOAD_NORMAL_PAR;
    packet.sourceID = SID_TC_RPW_INTERNAL;
    packet.sy_lfr_n_swf_l[0]    = (unsigned char) (spinbox_sy_lfr_n_swf_l->value() >> 8     );
    packet.sy_lfr_n_swf_l[1]    = (unsigned char) (spinbox_sy_lfr_n_swf_l->value()          );
    packet.sy_lfr_n_swf_p[0]    = (unsigned char) (spinbox_sy_lfr_n_swf_p->value() >> 8     );
    packet.sy_lfr_n_swf_p[1]    = (unsigned char) (spinbox_sy_lfr_n_swf_p->value()          );
    packet.sy_lfr_n_asm_p[0]    = (unsigned char) (spinbox_sy_lfr_n_asm_p->value() >> 8     );
    packet.sy_lfr_n_asm_p[1]    = (unsigned char) (spinbox_sy_lfr_n_asm_p->value()          );
    packet.sy_lfr_n_bp_p0       = (unsigned char) (spinbox_sy_lfr_n_bp_p0->value()          );
    packet.sy_lfr_n_bp_p1       = (unsigned char) (spinbox_sy_lfr_n_bp_p1->value()          );
    packet.sy_lfr_n_cwf_long_f3 = (unsigned char) (spinbox_sy_lfr_n_cwf_long_f3->value()    );
    packet.lfr_normal_parameters_spare = 0x00;

    paulCommonCRC->GetCRCAsTwoBytes((unsigned char*) &packet, crcAsTwoBytes,
                                   PACKET_LENGTH_TC_LFR_LOAD_NORMAL_PAR + CCSDS_TC_TM_PACKET_OFFSET - 2);
    packet.crc[0] = crcAsTwoBytes[0];
    packet.crc[1] = crcAsTwoBytes[1];

    emit WriteSPWSig((char*) &packet, PACKET_LENGTH_TC_LFR_LOAD_NORMAL_PAR + CCSDS_TC_TM_PACKET_OFFSET,
                     CCSDS_NODE_ADDRESS, CCSDS_USER_APP);
}

void ParameterDump::sendLoadBurst()
{
    Packet_TC_LFR_LOAD_BURST_SBM1_SBM2_PAR_t packet;
    unsigned char crcAsTwoBytes[2];

    packet.packetID[0] = (unsigned char) (TC_LFR_PACKET_ID >> 8);
    packet.packetID[1] = (unsigned char) (TC_LFR_PACKET_ID     );
    packet.packetSequenceControl[0] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL >> 8);
    packet.packetSequenceControl[1] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL     );
    packet.packetLength[0] = (unsigned char) (PACKET_LENGTH_TC_LFR_LOAD_BURST_PAR >> 8);
    packet.packetLength[1] = (unsigned char) (PACKET_LENGTH_TC_LFR_LOAD_BURST_PAR     );

    packet.ccsdsSecHeaderFlag_pusVersion_ack = TC_LFR_DATA_FIELD_HEADER0;
    packet.serviceType = TC_TYPE_DEFAULT;
    packet.serviceSubType = TC_SUBTYPE_LOAD_BURST_PAR;
    packet.sourceID = SID_TC_RPW_INTERNAL;
    packet.sy_lfr_bp_p0    = (unsigned char) (spinbox_sy_lfr_b_bp_p0->value()     );
    packet.sy_lfr_bp_p1    = (unsigned char) (spinbox_sy_lfr_b_bp_p1->value()     );

    paulCommonCRC->GetCRCAsTwoBytes((unsigned char*) &packet, crcAsTwoBytes,
                                   PACKET_LENGTH_TC_LFR_LOAD_BURST_PAR + CCSDS_TC_TM_PACKET_OFFSET - 2);
    packet.crc[0] = crcAsTwoBytes[0];
    packet.crc[1] = crcAsTwoBytes[1];

    emit WriteSPWSig((char*) &packet, PACKET_LENGTH_TC_LFR_LOAD_BURST_PAR + CCSDS_TC_TM_PACKET_OFFSET,
                     CCSDS_NODE_ADDRESS, CCSDS_USER_APP);
}

void ParameterDump::sendLoadSBM1()
{
    Packet_TC_LFR_LOAD_BURST_SBM1_SBM2_PAR_t packet;
    unsigned char crcAsTwoBytes[2];

    packet.packetID[0] = (unsigned char) (TC_LFR_PACKET_ID >> 8);
    packet.packetID[1] = (unsigned char) (TC_LFR_PACKET_ID     );
    packet.packetSequenceControl[0] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL >> 8);
    packet.packetSequenceControl[1] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL     );
    packet.packetLength[0] = (unsigned char) (PACKET_LENGTH_TC_LFR_LOAD_SBM1_PAR >> 8);
    packet.packetLength[1] = (unsigned char) (PACKET_LENGTH_TC_LFR_LOAD_SBM1_PAR     );

    packet.ccsdsSecHeaderFlag_pusVersion_ack = TC_LFR_DATA_FIELD_HEADER0;
    packet.serviceType = TC_TYPE_DEFAULT;
    packet.serviceSubType = TC_SUBTYPE_LOAD_SBM1_PAR;
    packet.sourceID = SID_TC_RPW_INTERNAL;
    packet.sy_lfr_bp_p0    = (unsigned char) (spinbox_sy_lfr_s1_bp_p0->value()     );
    packet.sy_lfr_bp_p1    = (unsigned char) (spinbox_sy_lfr_s1_bp_p1->value()     );

    paulCommonCRC->GetCRCAsTwoBytes((unsigned char*) &packet, crcAsTwoBytes,
                                   PACKET_LENGTH_TC_LFR_LOAD_SBM1_PAR + CCSDS_TC_TM_PACKET_OFFSET - 2);
    packet.crc[0] = crcAsTwoBytes[0];
    packet.crc[1] = crcAsTwoBytes[1];

    emit WriteSPWSig((char*) &packet, PACKET_LENGTH_TC_LFR_LOAD_SBM1_PAR + CCSDS_TC_TM_PACKET_OFFSET,
                     CCSDS_NODE_ADDRESS, CCSDS_USER_APP);
}

void ParameterDump::sendLoadSBM2()
{
    Packet_TC_LFR_LOAD_BURST_SBM1_SBM2_PAR_t packet;
    unsigned char crcAsTwoBytes[2];

    packet.packetID[0] = (unsigned char) (TC_LFR_PACKET_ID >> 8);
    packet.packetID[1] = (unsigned char) (TC_LFR_PACKET_ID     );
    packet.packetSequenceControl[0] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL >> 8);
    packet.packetSequenceControl[1] = (unsigned char) (TC_LFR_PACKET_SEQUENCE_CONTROL     );
    packet.packetLength[0] = (unsigned char) (PACKET_LENGTH_TC_LFR_LOAD_SBM2_PAR >> 8);
    packet.packetLength[1] = (unsigned char) (PACKET_LENGTH_TC_LFR_LOAD_SBM2_PAR     );

    packet.ccsdsSecHeaderFlag_pusVersion_ack = TC_LFR_DATA_FIELD_HEADER0;
    packet.serviceType = TC_TYPE_DEFAULT;
    packet.serviceSubType = TC_SUBTYPE_LOAD_SBM2_PAR;
    packet.sourceID = SID_TC_RPW_INTERNAL;
    packet.sy_lfr_bp_p0    = (unsigned char) (spinbox_sy_lfr_s2_bp_p0->value()     );
    packet.sy_lfr_bp_p1    = (unsigned char) (spinbox_sy_lfr_s2_bp_p1->value()     );

    paulCommonCRC->GetCRCAsTwoBytes((unsigned char*) &packet, crcAsTwoBytes,
                                   PACKET_LENGTH_TC_LFR_LOAD_SBM2_PAR + CCSDS_TC_TM_PACKET_OFFSET - 2);
    packet.crc[0] = crcAsTwoBytes[0];
    packet.crc[1] = crcAsTwoBytes[1];

    emit WriteSPWSig((char*) &packet, PACKET_LENGTH_TC_LFR_LOAD_SBM2_PAR + CCSDS_TC_TM_PACKET_OFFSET,
                     CCSDS_NODE_ADDRESS, CCSDS_USER_APP);
}

void ParameterDump::actionsSetEnabled(bool state)
{
    button_parameterDump->setEnabled(state);
    button_loadCommon->setEnabled(state);
    button_loadNormal->setEnabled(state);
    button_loadBurst->setEnabled(state);
    button_loadSBM1->setEnabled(state);
    button_loadSBM2->setEnabled(state);
}

//***********************
// k coefficients records

QString ParameterDump::buildPrefix()
{
    QTime time;
    QDate date;
    QString dateTime;
    QString prefix;

    date = QDate::currentDate();
    time = QTime::currentTime();

    dateTime =
            QString("%1").arg(date.year(),    4, 10, QChar('0')) + "_"
            + QString("%1").arg(date.month(), 2, 10, QChar('0')) + "_"
            + QString("%1").arg(date.day(),   2, 10, QChar('0')) + "_"
            + QString("%1").arg(time.hour(),  2, 10, QChar('0')) + "_"
            + QString("%1").arg(time.minute(),2, 10, QChar('0')) + "_"
            + QString("%1").arg(time.second(),2, 10, QChar('0'));

    prefix = defaultStorageDirectory + "/" + dateTime + "_" ;

    return prefix;
}

void ParameterDump::buildKCoefficientsFileName()
{
    if(this->kCoefficientsRecordFile->isOpen()) this->kCoefficientsRecordFile->close();
    this->kCoefficientsRecordFile->setFileName( buildPrefix() + "k_cofficients.data");
    if(this->kCoefficientsRecordFile->open(QIODevice::WriteOnly))
        this->kCoefficientsRecordFileStrm = new QTextStream(this->kCoefficientsRecordFile);
}

void ParameterDump::buildFBinsMasksFileName()
{
    if(this->fBinsMasksRecordFile->isOpen()) this->fBinsMasksRecordFile->close();
    this->fBinsMasksRecordFile->setFileName( buildPrefix() + "fbins_masks.data");
    if(this->fBinsMasksRecordFile->open(QIODevice::WriteOnly))
        this->fBinsMasksRecordFileStrm = new QTextStream(this->fBinsMasksRecordFile);
}


void ParameterDump::storeKCoefficients( int state )
{
    if (state == Qt::Checked)
    {
        chooseDir();
        kCoefficientsRecordFileEn = true;
    }
    else
    {
        if(this->kCoefficientsRecordFile->isOpen()) this->kCoefficientsRecordFile->close();
        kCoefficientsRecordFileEn = false;
    }
}

void ParameterDump::storeFBinsMasks( int state )
{
    if (state == Qt::Checked)
    {
        fBinsMasksRecordFileEn = true;
        chooseDir();
    }
    else
    {
        if(this->fBinsMasksRecordFile->isOpen()) this->fBinsMasksRecordFile->close();
        fBinsMasksRecordFileEn = false;
    }
}

void ParameterDump::chooseDir()
{
    QString tmpDefaultStorageDirectory;
    tmpDefaultStorageDirectory = QFileDialog::getExistingDirectory(this,
                                          "choose the directory",
                                          defaultStorageDirectory,
                                          QFileDialog::ShowDirsOnly);

    if( !tmpDefaultStorageDirectory.isEmpty() )
    {
        defaultStorageDirectory = tmpDefaultStorageDirectory;
    }
}

void ParameterDump::processKCoefficientsDump( TMPacketToRead *packet )
{
    if (kCoefficientsRecordFileEn == true)
    {
        processTMKCoefficientsDump( packet );
    }
}

void ParameterDump::processTMKCoefficientsDump( TMPacketToRead *packet )
{
    unsigned char pkt_cnt;
    unsigned char pkt_nr;
    unsigned char blk_nr;
    unsigned char k;
    unsigned char i;
    unsigned short freq;
    float kCoeff;
    unsigned char *charPtr;
    unsigned int offset;
    unsigned int offsetK;

    pkt_cnt = packet->Value[21];
    pkt_nr = packet->Value[22];
    blk_nr = packet->Value[23];

    printf("pkt_cnt = %d, pkt_nr = %d, nlk_nr = %d\n", pkt_cnt, pkt_nr, blk_nr);

    charPtr = (unsigned char*) &kCoeff;

    for (k=0; k<blk_nr; k++)
    {
        offset = 24 + k * 130;
        freq        = packet->Value[ offset ] * 256 + packet->Value[ offset + 1];
        *(this->kCoefficientsRecordFileStrm)
                << QString::number( freq );
        for (i=0; i<32; i++)
        {
            offsetK = offset + 4 * i + 2;   // 2 is for the frequency
            charPtr[0] = packet->Value[ offsetK + 3 ];
            charPtr[1] = packet->Value[ offsetK + 2 ];
            charPtr[2] = packet->Value[ offsetK + 1 ];
            charPtr[3] = packet->Value[ offsetK + 0 ];
            *(this->kCoefficientsRecordFileStrm)
                    << " " + QString::number( kCoeff, 'g' );
            printf("freq = %d, i = %d, kcoeff = %f\n", freq, i, kCoeff);
        }
        *(this->kCoefficientsRecordFileStrm)
                << endl;
    }
}

void ParameterDump::processFBinsMasksDump( TMPacketToRead *packet )
{
    unsigned int coeff;
    unsigned int i;
    unsigned char *fBinsMaskPtr;
    unsigned char currentCoeff;

    Packet_TM_LFR_PARAMETER_DUMP_t *parameterPacket;

    parameterPacket = (Packet_TM_LFR_PARAMETER_DUMP_t *) packet->Value;

    fBinsMaskPtr = (unsigned char*) parameterPacket->sy_lfr_fbins_f0_word1;

    buildFBinsMasksFileName();

    for (coeff=0; coeff< 4; coeff++)
    {
        *(this->fBinsMasksRecordFileStrm)
                << "sy_lfr_fbins_f0";
        for (i=0; i<4; i++)
        {
            currentCoeff = fBinsMaskPtr[ coeff * 4 + i ];
            *(this->fBinsMasksRecordFileStrm)
                    << "   " + QString::number( (currentCoeff & 0x80) >> 7 )
                    << " " + QString::number( (currentCoeff & 0x40) >> 6 )
                    << " " + QString::number( (currentCoeff & 0x20) >> 5 )
                    << " " + QString::number( (currentCoeff & 0x10) >> 4 )
                    << " " + QString::number( (currentCoeff & 0x08) >> 3 )
                    << " " + QString::number( (currentCoeff & 0x04) >> 2 )
                    << " " + QString::number( (currentCoeff & 0x02) >> 1 )
                    << " " + QString::number( (currentCoeff & 0x01) >> 0 );
        }
        *(this->fBinsMasksRecordFileStrm)
                << endl;
    }

    *(this->fBinsMasksRecordFileStrm)
            << endl;

    for (coeff=0; coeff< 4; coeff++)
    {
        *(this->fBinsMasksRecordFileStrm)
                << "sy_lfr_fbins_f1";
        for (i=0; i<4; i++)
        {
            currentCoeff = fBinsMaskPtr[ coeff * 4 + i + 16 ];
            *(this->fBinsMasksRecordFileStrm)
                    << "   " + QString::number( (currentCoeff & 0x80) >> 7 )
                    << " " + QString::number( (currentCoeff & 0x40) >> 6 )
                    << " " + QString::number( (currentCoeff & 0x20) >> 5 )
                    << " " + QString::number( (currentCoeff & 0x10) >> 4 )
                    << " " + QString::number( (currentCoeff & 0x08) >> 3 )
                    << " " + QString::number( (currentCoeff & 0x04) >> 2 )
                    << " " + QString::number( (currentCoeff & 0x02) >> 1 )
                    << " " + QString::number( (currentCoeff & 0x01) >> 0 );
        }
        *(this->fBinsMasksRecordFileStrm)
                << endl;
    }

    *(this->fBinsMasksRecordFileStrm)
            << endl;

    for (coeff=0; coeff< 4; coeff++)
    {
        *(this->fBinsMasksRecordFileStrm)
                << "sy_lfr_fbins_f2";
        for (i=0; i<4; i++)
        {
            currentCoeff = fBinsMaskPtr[ coeff * 4 + i + 32 ];
            *(this->fBinsMasksRecordFileStrm)
                    << "   " + QString::number( (currentCoeff & 0x80) >> 7 )
                    << " " + QString::number( (currentCoeff & 0x40) >> 6 )
                    << " " + QString::number( (currentCoeff & 0x20) >> 5 )
                    << " " + QString::number( (currentCoeff & 0x10) >> 4 )
                    << " " + QString::number( (currentCoeff & 0x08) >> 3 )
                    << " " + QString::number( (currentCoeff & 0x04) >> 2 )
                    << " " + QString::number( (currentCoeff & 0x02) >> 1 )
                    << " " + QString::number( (currentCoeff & 0x01) >> 0 );
        }
        *(this->fBinsMasksRecordFileStrm)
                << endl;
    }
}

void ParameterDump::readSettings()
{
    QSettings settings("ple", "parameterdump");
    defaultStorageDirectory = settings.value( "defaultStorageDirectory", QDir::homePath() ).toString();
}

void ParameterDump::writeSettings()
{
    QSettings settings("ple", "parameterdump");
    settings.setValue("defaultStorageDirectory", defaultStorageDirectory);
}
